/*
 * ******************************************************************************************
 *
 * File		: apb_wdt.h
 * Author	: GOWIN
 * Function	: Definition of APB Watchdog
 *
 * ******************************************************************************************
 */

#ifndef __APB_WDT_H__
#define __APB_WDT_H__


#ifdef __cplusplus
extern "C" {
#endif


// Includes ---------------------------------------------------------------------------------
#include <stddef.h>


/*****************************************************************************
 * Device Specific Peripheral Registers structures
 ****************************************************************************/
#define __RO          volatile const  /* 'read  only' permissions     */
#define __WO          volatile        /* 'write only' permissions     */
#define __RW          volatile        /* 'read/write' permissions     */


/*****************************************************************************
 * APB WDT (Watchdog timer)
 ****************************************************************************/
typedef struct
{
	__RO  unsigned int IDREV;                /* 0x00 ID and Revision Register */
	      unsigned int RESERVED[3];          /* 0x04~0x0C Reserved            */
	__RW  unsigned int CTRL;                 /* 0x10 Control Register         */
	__WO  unsigned int RESTART;              /* 0x14 Restart Register         */
	__WO  unsigned int WREN;                 /* 0x18 Write Enable Register    */
	__RW  unsigned int ST;                   /* 0x1C Status Register          */
} APB_WDT_RegDef;


/****** APB WDT Control Codes *****/

#define APB_WDT_CONTROL_Pos               0
#define APB_WDT_CONTROL_Msk               (0xFFUL << APB_WDT_CONTROL_Pos)

/*----- APB WDT Control Codes: Clock Source -----*/
#define APB_WDT_CLKSRC_Pos                0
#define APB_WDT_CLKSRC_Msk                (0x1UL << APB_WDT_CLKSRC_Pos)
#define APB_WDT_CLKSRC_APB                (0x1UL << APB_WDT_CLKSRC_Pos)     // System clock
#define APB_WDT_CLKSRC_EXTERNAL           (0x0UL << APB_WDT_CLKSRC_Pos)     // External clock (default)

/*----- APB WDT Time Period -----*/
#define APB_WDT_TIME_POW_2_6              (0x0UL)                           // APB WDT timer interval    64 clock periods.
#define APB_WDT_TIME_POW_2_8              (0x1UL)                           // APB WDT timer interval   256 clock periods.
#define APB_WDT_TIME_POW_2_10             (0x2UL)                           // APB WDT timer interval  1024 clock periods.
#define APB_WDT_TIME_POW_2_11             (0x3UL)                           // APB WDT timer interval  2048 clock periods.
#define APB_WDT_TIME_POW_2_12             (0x4UL)                           // APB WDT timer interval  4096 clock periods.
#define APB_WDT_TIME_POW_2_13             (0x5UL)                           // APB WDT timer interval  8192 clock periods.
#define APB_WDT_TIME_POW_2_14             (0x6UL)                           // APB WDT timer interval 16384 clock periods.
#define APB_WDT_TIME_POW_2_15             (0x7UL)                           // APB WDT timer interval 32768 clock periods.

/* General return codes */
#define APB_WDT_DRIVER_OK                 0 // Operation succeeded
#define APB_WDT_DRIVER_ERROR             -1 // Unspecified error
#define APB_WDT_DRIVER_ERROR_BUSY        -2 // Driver is busy
#define APB_WDT_DRIVER_ERROR_TIMEOUT     -3 // Timeout occurred
#define APB_WDT_DRIVER_ERROR_UNSUPPORTED -4 // Operation not supported
#define APB_WDT_DRIVER_ERROR_PARAMETER   -5 // Parameter error
#define APB_WDT_DRIVER_ERROR_SPECIFIC    -6 // Start of driver specific errors

/****** APB WDT specific error codes *****/
#define APB_WDT_ERROR_TIME_PREIOD         (APB_WDT_DRIVER_ERROR_SPECIFIC - 1)   // Specified time period not supported


/**************************************************************************
\brief APB WDT Status
***************************************************************************/
typedef struct _APB_WDT_STATUS
{
	unsigned int irq_expired  : 1;        		// APB WDT IRQ stage time expired (in APB WDT Interrupt)
} APB_WDT_STATUS;

/****** APB WDT Event *****/
#define APB_WDT_EVENT_INT_STAGE (1UL << 0)  	// APB WDT Interrupt event by IRQ stage time expired


/**************************************************************************
  \fn          void APB_WDT_SignalEvent (unsigned int event)
  \brief       Signal APB WDT Events.
  \param[in]   event  APB_WDT_events notification mask
  \return      none
***************************************************************************/
typedef void (*APB_WDT_SignalEvent_t) (unsigned int event);  // Pointer to APB_WDT_SignalEvent : Signal APB WDT Event.


/**************************************************************************
\brief APB WDT Device Driver Capabilities
***************************************************************************/
typedef struct _APB_WDT_CAPABILITIES
{
	unsigned int external_timer : 1; 	// Support using external timer as clock source.
	unsigned int irq_stage      : 1; 	// Support interrupt stage before APB WDT reset.
} APB_WDT_CAPABILITIES;


/* 0x10 Control Register */
#define APB_WDT_CTRL_RSTTIME_POW_2_7        0x000
#define APB_WDT_CTRL_RSTTIME_POW_2_8        0x100
#define APB_WDT_CTRL_RSTTIME_POW_2_9        0x200
#define APB_WDT_CTRL_RSTTIME_POW_2_10       0x300
#define APB_WDT_CTRL_RSTTIME_POW_2_11       0x400
#define APB_WDT_CTRL_RSTTIME_POW_2_12       0x500
#define APB_WDT_CTRL_RSTTIME_POW_2_13       0x600
#define APB_WDT_CTRL_RSTTIME_POW_2_14       0x700

#define APB_WDT_CTRL_INTTIME_POW_2_6        0x000
#define APB_WDT_CTRL_INTTIME_POW_2_8        0x010
#define APB_WDT_CTRL_INTTIME_POW_2_10       0x020
#define APB_WDT_CTRL_INTTIME_POW_2_11       0x030
#define APB_WDT_CTRL_INTTIME_POW_2_12       0x040
#define APB_WDT_CTRL_INTTIME_POW_2_13       0x050
#define APB_WDT_CTRL_INTTIME_POW_2_14       0x060
#define APB_WDT_CTRL_INTTIME_POW_2_15       0x070

#define APB_WDT_CTRL_INTTIME_POW_2_17       0x080
#define APB_WDT_CTRL_INTTIME_POW_2_19       0x090
#define APB_WDT_CTRL_INTTIME_POW_2_21       0x0A0
#define APB_WDT_CTRL_INTTIME_POW_2_23       0x0B0
#define APB_WDT_CTRL_INTTIME_POW_2_25       0x0C0
#define APB_WDT_CTRL_INTTIME_POW_2_27       0x0D0
#define APB_WDT_CTRL_INTTIME_POW_2_29       0x0E0
#define APB_WDT_CTRL_INTTIME_POW_2_31       0x0F0

#define APB_WDT_CTRL_RSTEN                  0x8
#define APB_WDT_CTRL_INTEN                  0x4
#define APB_WDT_CTRL_APBCLK                 0x2
#define APB_WDT_CTRL_EXTCLK                 0x0
#define APB_WDT_CTRL_EN                     0x1

/* 0x14 Restart Register */
#define APB_WDT_RESTART_NUM                 0xcafe

/* 0x18 Write Enable Register */
#define APB_WDT_WREN_NUM                    0x5aa5

/* 0x1C Status Register */
#define APB_WDT_ST_INTEXPIRED               0x1
#define APB_WDT_ST_INTEXPIRED_CLR           0x1   // w1c bit

// APB WDT flags
#define APB_WDT_FLAG_INITIALIZED            (1U << 0)
#define APB_WDT_FLAG_POWERED                (1U << 1)

// APB WDT information (Run-Time)
typedef struct _APB_WDT_INFO
{
	APB_WDT_SignalEvent_t   cb_event;       // Event callback
	unsigned int            flags;          // APB WDT driver flags
} APB_WDT_INFO;

// APB WDT resources definitions
typedef struct
{
	APB_WDT_CAPABILITIES     capabilities;  // Capabilities
	APB_WDT_RegDef           *reg;          // Pointer to APB WDT register
	APB_WDT_INFO             *info;         // Run-Time information
} const APB_WDT_RESOURCES;


#ifdef __cplusplus
}
#endif


#endif	// __APB_WDT_H__
