/*
 * ******************************************************************************************
 *
 * File		: apb_wdt_driver.c
 * Author	: GOWIN
 * Function	: Driver definition of APB Watchdog
 *
 * ******************************************************************************************
 */


// Includes ---------------------------------------------------------------------------------
#include "apb_wdt_driver.h"


// Definitions ------------------------------------------------------------------------------

/*****************************************************************************
  \fn          APB_WDT_CAPABILITIES apb_wdt_get_capabilities (APB_WDT_RESOURCES *apb_wdtx)
  \brief       Get driver capabilities
  \param[in]   apb_wdtx Pointer to APB WDT resources
  \return      APB_WDT_CAPABILITIES
******************************************************************************/
APB_WDT_CAPABILITIES apb_wdt_get_capabilities(APB_WDT_RESOURCES *apb_wdtx)
{
	return apb_wdtx->capabilities;
}

/*****************************************************************************
  \fn          int apb_wdt_initialize (APB_WDT_SignalEvent_t cb_event, APB_WDT_RESOURCES *apb_wdtx)
  \brief       Initialize APB WDT Interface.
  \param[in]   cb_event  Pointer to APB_WDT_SignalEvent
  \param[in]   apb_wdtx Pointer to APB WDT resources
  \return      execution_status
******************************************************************************/
int apb_wdt_initialize(APB_WDT_SignalEvent_t cb_event, APB_WDT_RESOURCES *apb_wdtx)
{
	if (apb_wdtx->info->flags & APB_WDT_FLAG_INITIALIZED)
	{
		// Driver is already initialized
		return APB_WDT_DRIVER_OK;
	}

	apb_wdtx->info->cb_event = cb_event;
	apb_wdtx->info->flags = APB_WDT_FLAG_INITIALIZED;

	return APB_WDT_DRIVER_OK;
}

/*****************************************************************************
  \fn          int apb_wdt_uninitialize (APB_WDT_RESOURCES *apb_wdtx)
  \brief       Uninitialized APB WDT Interface.
  \param[in]   apb_wdtx Pointer to APB WDT resources
  \return      execution_status
******************************************************************************/
int apb_wdt_uninitialize(APB_WDT_RESOURCES *apb_wdtx)
{
	// Reset APB WDT status flags
	apb_wdtx->info->flags = 0U;
	apb_wdtx->info->cb_event = NULL;

	return APB_WDT_DRIVER_OK;
}

/*****************************************************************************
  \fn          void apb_wdt_restart_timer (APB_WDT_RESOURCES *apb_wdtx)
  \brief       Restart APB WDT Timer
  \param[in]   apb_wdtx Pointer to APB WDT resources
  \return      none
******************************************************************************/
void apb_wdt_restart_timer(APB_WDT_RESOURCES *apb_wdtx)
{
	apb_wdtx->reg->WREN = APB_WDT_WREN_NUM;
	apb_wdtx->reg->RESTART = APB_WDT_RESTART_NUM;
}

/******************************************************************************
  \fn          void apb_wdt_clear_irq_status (APB_WDT_RESOURCES *apb_wdtx)
  \brief       Clear APB WDT IRQ Expired status
  \param[in]   apb_wdtx Pointer to APB WDT resources
  \return      none
*******************************************************************************/
void apb_wdt_clear_irq_status(APB_WDT_RESOURCES *apb_wdtx)
{
	apb_wdtx->reg->ST = APB_WDT_ST_INTEXPIRED_CLR;
}

/******************************************************************************
  \fn          int apb_wdt_control (unsigned int control, unsigned int arg, APB_WDT_RESOURCES *apb_wdtx)
  \brief       Control APB WDT Interface.
  \param[in]   control  Operation
  \param[in]   arg      Argument of operation (optional)
  \param[in]   apb_wdtx Pointer to APB WDT resources
  \return      common   execution_status and driver specific
  	  	  	  	  	    apb_wdt_execution_status
*******************************************************************************/
int apb_wdt_control(unsigned int control, unsigned int arg, APB_WDT_RESOURCES *apb_wdtx)
{
	if ((apb_wdtx->info->flags & APB_WDT_FLAG_INITIALIZED) == 0U)
	{
		// APB WDT is not initialized
		return APB_WDT_DRIVER_ERROR;
	}

	unsigned int time_period = arg;
	unsigned int ctrl = 0;

	// Enable APB WDT RST/INT stages.
	ctrl |= (APB_WDT_CTRL_RSTEN | APB_WDT_CTRL_INTEN);
	// Set RST time to minimum.
	ctrl |= APB_WDT_CTRL_RSTTIME_POW_2_7;

	switch (control & APB_WDT_CLKSRC_Msk)
	{
		case APB_WDT_CLKSRC_APB:
			ctrl |= APB_WDT_CTRL_APBCLK;
			break;
		case APB_WDT_CLKSRC_EXTERNAL:
			ctrl |= APB_WDT_CTRL_EXTCLK;
			break;
		default:
			break;
	}

	switch (time_period)
	{
		case APB_WDT_TIME_POW_2_6:
			ctrl |= APB_WDT_CTRL_INTTIME_POW_2_6;
			break;
		case APB_WDT_TIME_POW_2_8:
			ctrl |= APB_WDT_CTRL_INTTIME_POW_2_8;
			break;
		case APB_WDT_TIME_POW_2_10:
			ctrl |= APB_WDT_CTRL_INTTIME_POW_2_10;
			break;
		case APB_WDT_TIME_POW_2_11:
			ctrl |= APB_WDT_CTRL_INTTIME_POW_2_11;
			break;
		case APB_WDT_TIME_POW_2_12:
			ctrl |= APB_WDT_CTRL_INTTIME_POW_2_12;
			break;
		case APB_WDT_TIME_POW_2_13:
			ctrl |= APB_WDT_CTRL_INTTIME_POW_2_13;
			break;
		case APB_WDT_TIME_POW_2_14:
			ctrl |= APB_WDT_CTRL_INTTIME_POW_2_14;
			break;
		case APB_WDT_TIME_POW_2_15:
			ctrl |= APB_WDT_CTRL_INTTIME_POW_2_15;
			break;
		default:
			return APB_WDT_ERROR_TIME_PREIOD;
	}

	apb_wdt_restart_timer(apb_wdtx);
	apb_wdt_clear_irq_status(apb_wdtx);

	apb_wdtx->reg->WREN = APB_WDT_WREN_NUM;
	apb_wdtx->reg->CTRL = ctrl;

	return APB_WDT_DRIVER_OK;
}

/******************************************************************************
  \fn          void apb_wdt_enable (APB_WDT_RESOURCES *apb_wdtx)
  \brief       Enable APB WDT Timer
  \param[in]   apb_wdtx Pointer to APB WDT resources
  \return      none
*******************************************************************************/
void apb_wdt_enable(APB_WDT_RESOURCES *apb_wdtx)
{
	unsigned int ctrl = apb_wdtx->reg->CTRL;

	ctrl |= APB_WDT_CTRL_EN;
	apb_wdtx->reg->WREN = APB_WDT_WREN_NUM;
	apb_wdtx->reg->CTRL = ctrl;
}

/******************************************************************************
  \fn          void apb_wdt_disable (APB_WDT_RESOURCES *apb_wdtx)
  \brief       Disable APB WDT Timer
  \param[in]   apb_wdtx Pointer to APB WDT resources
  \return      none
*******************************************************************************/
void apb_wdt_disable(APB_WDT_RESOURCES *apb_wdtx)
{
	unsigned int ctrl = apb_wdtx->reg->CTRL;

	ctrl &= ~APB_WDT_CTRL_EN;
	apb_wdtx->reg->WREN = APB_WDT_WREN_NUM;
	apb_wdtx->reg->CTRL = ctrl;
}

/*******************************************************************************
  \fn          APB_WDT_STATUS apb_wdt_get_status (APB_WDT_RESOURCES *apb_wdtx)
  \brief       Get APB WDT status.
  \param[in]   apb_wdtx Pointer to APB WDT resources
  \return      APB WDT status APB_WDT_STATUS
********************************************************************************/
APB_WDT_STATUS apb_wdt_get_status(APB_WDT_RESOURCES *apb_wdtx)
{
	APB_WDT_STATUS status = {0};

	unsigned int status_reg = apb_wdtx->reg->ST;

	if( status_reg & APB_WDT_ST_INTEXPIRED )
	{
		status.irq_expired = 1;
	}

	return status;
}
